<?php

// must be run within Dokuwiki
if(!defined('DOKU_INC')) die();
 
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
if(!defined('WIKI_PAPERS_PLUGIN')) define('WIKI_PAPERS_PLUGIN', DOKU_PLUGIN.'wiki_papers/');
require_once(WIKI_PAPERS_PLUGIN.'search.php');
require_once(WIKI_PAPERS_PLUGIN.'functions.php');
require_once(WIKI_PAPERS_PLUGIN.'draw.php');


class syntax_plugin_wiki_papers extends DokuWiki_Syntax_Plugin
{
	function getInfo()
	{
		return array
		(
			'author' => 'Rafał Gibas',
			'email'  => 'coolgibas@gmail.com',
			'date'   => '2009-03-dd',
			'name'   => 'Wiki Papers Plugin',
			'desc'   => 'Writing publication graphs and statistics',
			'url'    => 'https://ai.ia.agh.edu.pl/wiki/pl:miw:2009:miw09_wiki_papers',
		);
	}

	function getType()
	{
		return 'container';
	}
	
    function getAllowedTypes() { return array('disabled'); }  
	
	function getPType()
	{
		return 'block';
	}

	function getSort()
	{	
		return 0;
	}

	function connectTo($mode)
	{
		$this->Lexer->addEntryPattern('<papers>(?=.*</papers>)',$mode,'plugin_wiki_papers');
		//$this->Lexer->addPattern('\[\[[^\[\]]*\]\]',$mode,'plugin_wiki_papers');
		//$this->Lexer->addPattern('DBLP:[0-9]+ ',$mode,'plugin_wiki_papers');
	}
 
	function postConnect()
	{
		$this->Lexer->addExitPattern('</papers>', 'plugin_wiki_papers');
	}

	function handle($match, $state, $pos, &$handler)
	{
		$matchstr = $match . "";
		
		switch ($state){
		case DOKU_LEXER_ENTER:
			return array();
		case DOKU_LEXER_EXIT:
			return array();
		case DOKU_LEXER_UNMATCHED:
		
			$m = "";
			$options = split(";", strtoupper(cut_string_between($match, "options:", ";;")));
			list($HIDEAUTHOR, $SHOWINFO, $SUM, $TABLE, $TEXT, $CHART) = array(	in_array('HIDEAUTHOR', $options), in_array('SHOWINFO', $options),
																				in_array('SUM', $options), in_array('TABLE', $options),
																				in_array('TEXT', $options), in_array('CHART', $options));
			// HIDEAUTHOR SHOWINFO SUM TABLE TEXT CHART NOHEADERS
			
			$h_a = cut_string_between($match, "headers:", ";;");
			$headers_alt = ($h_a === "") ? null : split(";", $h_a);
			$title = cut_string_between($match, "title:", ";;");
			
			$bibls = array("DBLP", "BPP");//, "CiteSeerX");
			
			$match = " " . $match;
			$rows = explode("author:", $match);
			array_shift($rows);	// Pierwszy element to śmieci
			
			$table = array();
			foreach ($rows as $rows_key => $row){
				$table[$rows_key] = array();
				$row .= " ";
				$cols = explode(";;", $row);
				array_pop($cols);	// Ostatni element to śmieci
				
				$autstr = array_shift($cols);
				list($author, $url) = split_string($autstr);
				
				$ainfo = $author;
				if ($url !== "") $ainfo = "<a class=\"urlextern\" href=\"$url\">$author</a>";
				array_push($table[$rows_key], array("author", $ainfo, $author));
				
				foreach ($cols as $cols_key => $col){
					$bibl = str_replace("\n", "", str_replace(" ", "", cut_string_between($col, "", ":")));
					list($opts, $url, $year, $to_year) = split_string($col);
					$ystr = (isset($to_year)) ? $year . "-" . $to_year : $year;
					$info = "$bibl:$opts:$ystr";
					if ("$url$year$opts" == ""){
						array_push($table[$rows_key], array($info, "Err: No parameters", 0));
						continue;
					}
					
					$n = 0;
					if (in_array($bibl, $bibls)){
						list($number, $url) = getNumberOfPublications($bibl, $opts, $url, $author, $year, $to_year);
						//if(isset($to_year)) $year= $year . "-" . $to_year;
						$n = is_numeric($number) ? $number : 0;
						if ($url !== "" && !in_array("NOURL", $options)){
							$result	= "<a class=\"urlextern\" href=\"$url\">"
									. ((in_array("SHOWINFO", $options)) ? "$info=" : "")
									. "$number</a>";
						} else $result = $number;
					} else {
						$result = "Unknown bibliography \"$bibl\"";
					}
					array_push($table[$rows_key], array("$info", $result, $n));
				}
			}
			
		// Display
			if (in_array("CODE", $options)){
				$m .= "<pre class=\"code\">" . htmlentities("<papers>$matchstr</papers>") . "</pre>";
			}
			
			if (in_array("TEXT", $options) || !in_array("TABLE", $options) && !in_array("CHART", $options)){
				$m .= draw_text($table, $options);
			}
			
			if (in_array("TABLE", $options)){
				$m .= draw_table($table, $options, $headers_alt, $title);
			}
			
			$image = null;
			if (in_array("CHART", $options)){
				$image = draw_chart($table, $options, $matchstr, $title);
			}
			if (isset($image)) return array($m, $image);
			return array($m);
		default:
			return array();
		}
		return array('error');
	}

	function render($mode, &$renderer, $data)
	{
		if($mode == 'xhtml')
		{
			if($data != array('error'))
			{
				list($match, $image) = $data;
				if (isset($match)) $renderer -> doc .= $match;
				if (isset($image)) $renderer -> doc .= $renderer -> 	internalmedia($image);
			}
			else
			{
				$renderer->doc .= "<p>Parse error</p>";
			}
			return true;
		}
		return false;
	}

}

?>
